@@ -25,7 +25,7 @@ from octaviaclient.tests.unit.osc.v2 import fakes
 class TestPool(fakes.TestOctaviaClient):
 
     def setUp(self):
-        super(TestPool, self).setUp()
+        super().setUp()
 
         self._po = fakes.createFakeResource('pool')
         self.pool_info = copy.deepcopy(attr_consts.POOL_ATTRS)
@@ -41,7 +41,7 @@ class TestPool(fakes.TestOctaviaClient):
 class TestPoolList(TestPool):
 
     def setUp(self):
-        super(TestPoolList, self).setUp()
+        super().setUp()
         self.datalist = (tuple(
             attr_consts.POOL_ATTRS[k] for k in self.columns
         ),)
@@ -58,11 +58,67 @@ class TestPoolList(TestPool):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
 
+    def test_pool_list_with_tags(self):
+        arglist = ['--tags', 'foo,bar']
+        verifylist = [('tags', ['foo', 'bar'])]
+        expected_attrs = {
+            'tags': ['foo', 'bar']
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_list.assert_called_with(**expected_attrs)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_pool_list_with_any_tags(self):
+        arglist = ['--any-tags', 'foo,bar']
+        verifylist = [('any_tags', ['foo', 'bar'])]
+        expected_attrs = {
+            'tags-any': ['foo', 'bar']
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_list.assert_called_with(**expected_attrs)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_pool_list_with_not_tags(self):
+        arglist = ['--not-tags', 'foo,bar']
+        verifylist = [('not_tags', ['foo', 'bar'])]
+        expected_attrs = {
+            'not-tags': ['foo', 'bar']
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_list.assert_called_with(**expected_attrs)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_pool_list_with_not_any_tags(self):
+        arglist = ['--not-any-tags', 'foo,bar']
+        verifylist = [('not_any_tags', ['foo', 'bar'])]
+        expected_attrs = {
+            'not-tags-any': ['foo', 'bar']
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_list.assert_called_with(**expected_attrs)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
 
 class TestPoolDelete(TestPool):
 
     def setUp(self):
-        super(TestPoolDelete, self).setUp()
+        super().setUp()
         self.cmd = pool.DeletePool(self.app, None)
 
     def test_pool_delete(self):
@@ -99,7 +155,6 @@ class TestPoolDelete(TestPool):
         verifylist = [
             ('pool', 'unknown_pool')
         ]
-        self.api_mock.pool_list.return_value = {'pools': []}
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                           parsed_args)
@@ -109,7 +164,7 @@ class TestPoolDelete(TestPool):
 class TestPoolCreate(TestPool):
 
     def setUp(self):
-        super(TestPoolCreate, self).setUp()
+        super().setUp()
         self.api_mock.pool_create.return_value = {
             'pool': self.pool_info}
         lb_client = self.app.client_manager
@@ -128,7 +183,11 @@ class TestPoolCreate(TestPool):
                    '--tls-container-ref', self._po.tls_container_ref,
                    '--ca-tls-container-ref', self._po.ca_tls_container_ref,
                    '--crl-container-ref', self._po.crl_container_ref,
-                   '--tls-ciphers', self._po.tls_ciphers]
+                   '--tls-ciphers', self._po.tls_ciphers,
+                   '--tls-version', self._po.tls_versions[0],
+                   '--tls-version', self._po.tls_versions[1],
+                   '--alpn-protocol', self._po.alpn_protocols[0],
+                   '--alpn-protocol', self._po.alpn_protocols[1]]
 
         verifylist = [
             ('loadbalancer', 'mock_lb_id'),
@@ -139,7 +198,31 @@ class TestPoolCreate(TestPool):
             ('tls_container_ref', self._po.tls_container_ref),
             ('ca_tls_container_ref', self._po.ca_tls_container_ref),
             ('crl_container_ref', self._po.crl_container_ref),
-            ('tls_ciphers', self._po.tls_ciphers)
+            ('tls_ciphers', self._po.tls_ciphers),
+            ('tls_versions', self._po.tls_versions),
+            ('alpn_protocols', self._po.alpn_protocols),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.api_mock.pool_create.assert_called_with(
+            json={'pool': self.pool_info})
+
+    @mock.patch('octaviaclient.osc.v2.utils.get_pool_attrs')
+    def test_pool_create_with_tag(self, mock_attrs):
+        mock_attrs.return_value = self.pool_info
+        arglist = ['--loadbalancer', 'mock_lb_id',
+                   '--name', self._po.name,
+                   '--protocol', 'HTTP',
+                   '--lb-algorithm', 'ROUND_ROBIN',
+                   '--tag', 'foo']
+
+        verifylist = [
+            ('loadbalancer', 'mock_lb_id'),
+            ('name', self._po.name),
+            ('protocol', 'HTTP'),
+            ('lb_algorithm', 'ROUND_ROBIN'),
+            ('tags', ['foo'])
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -181,7 +264,7 @@ class TestPoolCreate(TestPool):
 class TestPoolShow(TestPool):
 
     def setUp(self):
-        super(TestPoolShow, self).setUp()
+        super().setUp()
         self.api_mock.pool_show.return_value = self.pool_info
         lb_client = self.app.client_manager
         lb_client.load_balancer = self.api_mock
@@ -202,7 +285,7 @@ class TestPoolShow(TestPool):
 class TestPoolSet(TestPool):
 
     def setUp(self):
-        super(TestPoolSet, self).setUp()
+        super().setUp()
         self.cmd = pool.SetPool(self.app, None)
 
     def test_pool_set(self):
@@ -212,11 +295,17 @@ class TestPoolSet(TestPool):
         arglist = [self._po.id, '--name', 'new_name', '--tls-container-ref',
                    new_tls_id, '--ca-tls-container-ref', new_ca_id,
                    '--crl-container-ref', new_crl_id, '--enable-tls',
-                   '--tls-ciphers', self._po.tls_ciphers]
+                   '--tls-ciphers', self._po.tls_ciphers,
+                   '--tls-version', self._po.tls_versions[0],
+                   '--tls-version', self._po.tls_versions[1],
+                   '--alpn-protocol', self._po.alpn_protocols[0],
+                   '--alpn-protocol', self._po.alpn_protocols[1]]
         verifylist = [
             ('pool', self._po.id),
             ('name', 'new_name'),
-            ('tls_ciphers', self._po.tls_ciphers)
+            ('tls_ciphers', self._po.tls_ciphers),
+            ('tls_versions', self._po.tls_versions),
+            ('alpn_protocols', self._po.alpn_protocols)
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.cmd.take_action(parsed_args)
@@ -226,7 +315,11 @@ class TestPoolSet(TestPool):
                                         'ca_tls_container_ref': new_ca_id,
                                         'crl_container_ref': new_crl_id,
                                         'tls_enabled': True,
-                                        'tls_ciphers': self._po.tls_ciphers}})
+                                        'tls_ciphers': self._po.tls_ciphers,
+                                        'tls_versions': self._po.tls_versions,
+                                        'alpn_protocols':
+                                            self._po.alpn_protocols,
+                                        }})
 
     @mock.patch('osc_lib.utils.wait_for_status')
     def test_pool_set_wait(self, mock_wait):
@@ -246,14 +339,51 @@ class TestPoolSet(TestPool):
             sleep_time=mock.ANY,
             status_field='provisioning_status')
 
+    def test_pool_set_tag(self):
+        self.api_mock.pool_show.return_value = {
+            'tags': ['foo']
+        }
+        arglist = [self._po.id, '--tag', 'bar']
+        verifylist = [
+            ('pool', self._po.id),
+            ('tags', ['bar']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_set.assert_called_once()
+        kwargs = self.api_mock.pool_set.mock_calls[0][2]
+        tags = kwargs['json']['pool']['tags']
+        self.assertEqual(2, len(tags))
+        self.assertIn('foo', tags)
+        self.assertIn('bar', tags)
+
+    def test_pool_set_tag_no_tag(self):
+        self.api_mock.pool_show.return_value = {
+            'tags': ['foo']
+        }
+        arglist = [self._po.id, '--tag', 'bar', '--no-tag']
+        verifylist = [
+            ('pool', self._po.id),
+            ('tags', ['bar']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_set.assert_called_once_with(
+            self._po.id,
+            json={"pool": {"tags": ["bar"]}})
+
 
 class TestPoolUnset(TestPool):
     PARAMETERS = ('name', 'description', 'ca_tls_container_ref',
                   'crl_container_ref', 'session_persistence',
-                  'tls_container_ref')
+                  'tls_container_ref', 'tls_versions', 'tls_ciphers')
 
     def setUp(self):
-        super(TestPoolUnset, self).setUp()
+        super().setUp()
         self.cmd = pool.UnsetPool(self.app, None)
 
     def test_pool_unset_name(self):
@@ -277,6 +407,12 @@ class TestPoolUnset(TestPool):
     def test_pool_unset_tls_container_ref(self):
         self._test_pool_unset_param('tls_container_ref')
 
+    def test_pool_unset_tls_versions(self):
+        self._test_pool_unset_param('tls_versions')
+
+    def test_pool_unset_tls_ciphers(self):
+        self._test_pool_unset_param('tls_ciphers')
+
     def _test_pool_unset_param(self, param):
         self.api_mock.pool_set.reset_mock()
         arg_param = param.replace('_', '-') if '_' in param else param
@@ -337,3 +473,41 @@ class TestPoolUnset(TestPool):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.cmd.take_action(parsed_args)
         self.api_mock.pool_set.assert_not_called()
+
+    def test_pool_unset_tag(self):
+        self.api_mock.pool_set.reset_mock()
+        self.api_mock.pool_show.return_value = {
+            'tags': ['foo', 'bar']
+        }
+
+        arglist = [self._po.id, '--tag', 'foo']
+        verifylist = [
+            ('pool', self._po.id),
+            ('tags', ['foo']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_set.assert_called_once_with(
+            self._po.id,
+            json={"pool": {"tags": ["bar"]}})
+
+    def test_pool_unset_all_tag(self):
+        self.api_mock.pool_set.reset_mock()
+        self.api_mock.pool_show.return_value = {
+            'tags': ['foo', 'bar']
+        }
+
+        arglist = [self._po.id, '--all-tag']
+        verifylist = [
+            ('pool', self._po.id),
+            ('all_tag', True),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.pool_set.assert_called_once_with(
+            self._po.id,
+            json={"pool": {"tags": []}})
